import marshmallow as ma
from idutils import normalize_pid
from marshmallow import validate
from marshmallow.exceptions import ValidationError


class NRIdentifierSchema(ma.Schema):
    scheme = ma.fields.String(
        required=True,
    )
    identifier = ma.fields.String(required=True)

    @ma.post_load
    def normalize_identifier(self, value, *args, **kwargs):
        try:
            value["identifier"] = normalize_pid(
                value["identifier"], value["scheme"].lower()
            )
        except:
            raise ValidationError(f"Invalid {value['scheme']} value {value['identifier']}")

        return value

    @ma.pre_load
    def remove_url(self, value, *args, **kwargs):
        if not value or not isinstance(value, dict):
            return value
        url = value.pop("url", None)
        if url and not value.get("identifier"):
            value["identifier"] = url
        return value


class NRObjectIdentifierSchema(NRIdentifierSchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(["DOI", "Handle", "ISBN", "ISSN", "RIV"])
        ],  # RIV is not normalized, others are
    )


class NRAuthorityIdentifierSchema(NRIdentifierSchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                [
                    "orcid",  # normalized
                    "scopusID",
                    "researcherID",
                    "czenasAutID",
                    "vedidk",
                    "institutionalID",
                    "ISNI",
                    "ROR",
                    "ICO",
                    "DOI",  # normalized
                ]
            )
        ],
    )


class NRSystemIdentifierSchema(NRIdentifierSchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                ["nusl", "nuslOAI", "originalRecordOAI", "catalogueSysNo", "nrOAI"]
            )
        ],
    )

class NROrganizationIdentifierSchema(NRIdentifierSchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                [

                    "ISNI",
                    "ROR",
                    "ICO",
                    "DOI",  # normalized
                ]
            )
        ],
    )

class NRPersonIdentifierSchema(NRIdentifierSchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                [
                    "orcid",  # normalized
                    "scopusID",
                    "researcherID",
                    "czenasAutID",
                    "vedidk",
                    "institutionalID",
                    "ISNI",

                ]
            )
        ],
    )



from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links
from invenio_records_resources.services.records.components import DataComponent
from oarepo_runtime.services.config.service import PermissionsPresetsConfigMixin

from nr_metadata.common.records.api import CommonRecord
from nr_metadata.common.services.records.permissions import CommonPermissionPolicy
from nr_metadata.common.services.records.results import (
    CommonRecordItem,
    CommonRecordList,
)
from nr_metadata.common.services.records.schema_common import NRCommonRecordSchema
from nr_metadata.common.services.records.search import CommonSearchOptions


class CommonServiceConfig(PermissionsPresetsConfigMixin, InvenioRecordServiceConfig):
    """CommonRecord service config."""

    result_item_cls = CommonRecordItem

    result_list_cls = CommonRecordList

    PERMISSIONS_PRESETS = ["everyone"]

    url_prefix = "/nr-metadata-common/"

    base_permission_policy_cls = CommonPermissionPolicy

    schema = NRCommonRecordSchema

    search = CommonSearchOptions

    record_cls = CommonRecord

    service_id = "common"

    components = [
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        DataComponent,
    ]

    model = "nr_metadata.common"

    @property
    def links_item(self):
        return {
            "self": RecordLink("{+api}/nr-metadata-common/{id}"),
            "self_html": RecordLink("{+ui}/nr-metadata-common/{id}"),
        }

    @property
    def links_search(self):
        return {
            **pagination_links("{+api}/nr-metadata-common/{?args*}"),
        }



from oarepo_runtime.records.dumpers import SearchDumper

from nr_metadata.documents.records.dumpers.edtf import DocumentsEDTFIntervalDumperExt
from nr_metadata.documents.records.dumpers.multilingual import (
    MultilingualSearchDumperExt,
)


class DocumentsDumper(SearchDumper):
    """DocumentsRecord opensearch dumper."""

    extensions = [MultilingualSearchDumperExt(), DocumentsEDTFIntervalDumperExt()]



from oarepo_runtime.services.results import RecordItem, RecordList


class DataRecordItem(RecordItem):
    """DataRecord record item."""

    components = [*RecordItem.components]


class DataRecordList(RecordList):
    """DataRecord record list."""

    components = [*RecordList.components]



import marshmallow as ma
from marshmallow import Schema
from marshmallow import fields as ma_fields
from marshmallow.validate import OneOf
from oarepo_runtime.services.schema.i18n_ui import (
    I18nStrUIField,
    MultilingualLocalizedUIField,
    MultilingualUIField,
)
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.ui import (
    InvenioUISchema,
    LocalizedDate,
    LocalizedEDTF,
)

from nr_metadata.common.services.records.ui_schema_datatypes import (
    NRAccessRightsVocabularyUISchema,
    NRAffiliationVocabularyUISchema,
    NRContributorTypeVocabularyUISchema,
    NREventUISchema,
    NRFundingReferenceUISchema,
    NRGeoLocationUISchema,
    NRLanguageVocabularyUISchema,
    NRRelatedItemUISchema,
    NRResourceTypeVocabularyUISchema,
    NRRightsVocabularyUISchema,
    NRSeriesUISchema,
    NRSubjectCategoryVocabularyUISchema,
    NRSubjectUISchema,
)
from nr_metadata.ui_schema.identifiers import (
    NRObjectIdentifierUISchema,
    NROrganizationIdentifierUISchema,
    NRPersonIdentifierUISchema,
    NRSystemIdentifierUISchema,
)
from nr_metadata.ui_schema.subjects import NRSubjectListField


class NRCommonRecordUISchema(InvenioUISchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma_fields.Nested(lambda: NRCommonMetadataUISchema())


class NRCommonMetadataUISchema(Schema):
    class Meta:
        unknown = ma.RAISE

    abstract = MultilingualUIField(I18nStrUIField())

    accessRights = ma_fields.Nested(lambda: NRAccessRightsVocabularyUISchema())

    accessibility = MultilingualLocalizedUIField(I18nStrUIField())

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesUISchema())
    )

    contributors = ma_fields.List(ma_fields.Nested(lambda: NRContributorUISchema()))

    creators = ma_fields.List(
        ma_fields.Nested(lambda: NRCreatorUISchema()), required=True
    )

    dateAvailable = LocalizedDate()

    dateIssued = LocalizedEDTF()

    events = ma_fields.List(ma_fields.Nested(lambda: NREventUISchema()))

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: NRFundingReferenceUISchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationUISchema()))

    languages = ma_fields.List(ma_fields.Nested(lambda: NRLanguageVocabularyUISchema()))

    methods = MultilingualUIField(I18nStrUIField())

    notes = ma_fields.List(ma_fields.String())

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierUISchema())
    )

    originalRecord = ma_fields.String()

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemUISchema()))

    resourceType = ma_fields.Nested(
        lambda: NRResourceTypeVocabularyUISchema(), required=True
    )

    rights = ma_fields.Nested(lambda: NRRightsVocabularyUISchema())

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesUISchema()))

    subjectCategories = ma_fields.List(
        ma_fields.Nested(lambda: NRSubjectCategoryVocabularyUISchema())
    )

    subjects = NRSubjectListField(ma_fields.Nested(lambda: NRSubjectUISchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierUISchema())
    )

    technicalInfo = MultilingualUIField(I18nStrUIField())

    title = ma_fields.String(required=True)

    version = ma_fields.String()


class AdditionalTitlesUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    title = I18nStrUIField(required=True)

    titleType = ma_fields.String(
        required=True,
        validate=[OneOf(["translatedTitle", "alternativeTitle", "subtitle", "other"])],
    )


class NRContributorOrganizationUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    authorityIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NROrganizationIdentifierUISchema())
    )

    contributorType = ma_fields.Nested(lambda: NRContributorTypeVocabularyUISchema())

    fullName = ma_fields.String(required=True)

    nameType = ma_fields.String(validate=[OneOf(["Organizational"])])


class NRContributorPersonUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma_fields.List(
        ma_fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRPersonIdentifierUISchema())
    )

    contributorType = ma_fields.Nested(lambda: NRContributorTypeVocabularyUISchema())

    familyName = ma_fields.String()

    fullName = ma_fields.String(required=True)

    givenName = ma_fields.String()

    nameType = ma_fields.String(validate=[OneOf(["Personal"])])


class NRContributorUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma_fields.List(
        ma_fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NROrganizationIdentifierUISchema())
    )

    contributorType = ma_fields.Nested(lambda: NRContributorTypeVocabularyUISchema())

    familyName = ma_fields.String()

    fullName = ma_fields.String(required=True)

    givenName = ma_fields.String()

    nameType = ma_fields.String(validate=[OneOf(["Organizational"])])


class NRCreatorUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma_fields.List(
        ma_fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NROrganizationIdentifierUISchema())
    )

    familyName = ma_fields.String()

    fullName = ma_fields.String(required=True)

    givenName = ma_fields.String()

    nameType = ma_fields.String(validate=[OneOf(["Organizational"])])



"""Facet definitions."""

from invenio_records_resources.services.records.facets import TermsFacet
from oarepo_runtime.i18n import lazy_gettext as _
from oarepo_runtime.services.facets.date import DateTimeFacet
from oarepo_runtime.services.facets.nested_facet import NestedLabeledFacet
from oarepo_vocabularies.services.facets import (
    HierarchyVocabularyFacet,
    VocabularyFacet,
)

metadata_abstract_cs = TermsFacet(
    field="metadata.abstract.cs.keyword", label=_("metadata/abstract.label")
)

metadata_abstract_en = TermsFacet(
    field="metadata.abstract.en.keyword", label=_("metadata/abstract.label")
)

metadata_abstract_lang = NestedLabeledFacet(
    path="metadata.abstract",
    nested_facet=TermsFacet(
        field="metadata.abstract.lang", label=_("metadata/abstract/lang.label")
    ),
)

metadata_accessRights = VocabularyFacet(
    field="metadata.accessRights",
    label=_("metadata/accessRights.label"),
    vocabulary="access-rights",
)

metadata_accessibility_cs = TermsFacet(
    field="metadata.accessibility.cs.keyword", label=_("metadata/accessibility.label")
)

metadata_accessibility_en = TermsFacet(
    field="metadata.accessibility.en.keyword", label=_("metadata/accessibility.label")
)

metadata_accessibility_lang = NestedLabeledFacet(
    path="metadata.accessibility",
    nested_facet=TermsFacet(
        field="metadata.accessibility.lang",
        label=_("metadata/accessibility/lang.label"),
    ),
)

metadata_additionalTitles_title_cs = TermsFacet(
    field="metadata.additionalTitles.title.cs.keyword",
    label=_("metadata/additionalTitles/title.label"),
)

metadata_additionalTitles_title_en = TermsFacet(
    field="metadata.additionalTitles.title.en.keyword",
    label=_("metadata/additionalTitles/title.label"),
)

metadata_additionalTitles_title_lang = NestedLabeledFacet(
    path="metadata.additionalTitles.title",
    nested_facet=TermsFacet(
        field="metadata.additionalTitles.title.lang",
        label=_("metadata/additionalTitles/title/lang.label"),
    ),
)

metadata_additionalTitles_titleType = TermsFacet(
    field="metadata.additionalTitles.titleType",
    label=_("metadata/additionalTitles/titleType.label"),
)

metadata_contributors_affiliations = HierarchyVocabularyFacet(
    field="metadata.contributors.affiliations",
    label=_("metadata/contributors/affiliations.label"),
    vocabulary="institutions",
)

metadata_contributors_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.contributors.authorityIdentifiers.identifier",
    label=_("metadata/contributors/authorityIdentifiers/identifier.label"),
)

metadata_contributors_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.contributors.authorityIdentifiers.scheme",
    label=_("metadata/contributors/authorityIdentifiers/scheme.label"),
)

metadata_contributors_contributorType = VocabularyFacet(
    field="metadata.contributors.contributorType",
    label=_("metadata/contributors/contributorType.label"),
    vocabulary="contributor-types",
)

metadata_contributors_familyName = TermsFacet(
    field="metadata.contributors.familyName",
    label=_("metadata/contributors/familyName.label"),
)

metadata_contributors_fullName = TermsFacet(
    field="metadata.contributors.fullName",
    label=_("metadata/contributors/fullName.label"),
)

metadata_contributors_givenName = TermsFacet(
    field="metadata.contributors.givenName",
    label=_("metadata/contributors/givenName.label"),
)

metadata_contributors_nameType = TermsFacet(
    field="metadata.contributors.nameType",
    label=_("metadata/contributors/nameType.label"),
)

metadata_creators_affiliations = HierarchyVocabularyFacet(
    field="metadata.creators.affiliations",
    label=_("metadata/creators/affiliations.label"),
    vocabulary="institutions",
)

metadata_creators_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.creators.authorityIdentifiers.identifier",
    label=_("metadata/creators/authorityIdentifiers/identifier.label"),
)

metadata_creators_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.creators.authorityIdentifiers.scheme",
    label=_("metadata/creators/authorityIdentifiers/scheme.label"),
)

metadata_creators_familyName = TermsFacet(
    field="metadata.creators.familyName", label=_("metadata/creators/familyName.label")
)

metadata_creators_fullName = TermsFacet(
    field="metadata.creators.fullName", label=_("metadata/creators/fullName.label")
)

metadata_creators_givenName = TermsFacet(
    field="metadata.creators.givenName", label=_("metadata/creators/givenName.label")
)

metadata_creators_nameType = TermsFacet(
    field="metadata.creators.nameType", label=_("metadata/creators/nameType.label")
)

metadata_dateAvailable = DateTimeFacet(
    field="metadata.dateAvailable", label=_("metadata/dateAvailable.label")
)

metadata_dateIssued = DateTimeFacet(
    field="metadata.dateIssued", label=_("metadata/dateIssued.label")
)

metadata_dateValidTo = DateTimeFacet(
    field="metadata.dateValidTo", label=_("metadata/dateValidTo.label")
)

metadata_dateWithdrawn_dateInformation = TermsFacet(
    field="metadata.dateWithdrawn.dateInformation",
    label=_("metadata/dateWithdrawn/dateInformation.label"),
)

metadata_dateWithdrawn_type = DateTimeFacet(
    field="metadata.dateWithdrawn.type", label=_("metadata/dateWithdrawn/type.label")
)

metadata_events_eventLocation_country = VocabularyFacet(
    field="metadata.events.eventLocation.country",
    label=_("metadata/events/eventLocation/country.label"),
    vocabulary="countries",
)

metadata_events_eventLocation_place = TermsFacet(
    field="metadata.events.eventLocation.place",
    label=_("metadata/events/eventLocation/place.label"),
)

metadata_fundingReferences_funder = VocabularyFacet(
    field="metadata.fundingReferences.funder",
    label=_("metadata/fundingReferences/funder.label"),
    vocabulary="funders",
)

metadata_fundingReferences_projectID = TermsFacet(
    field="metadata.fundingReferences.projectID",
    label=_("metadata/fundingReferences/projectID.label"),
)

metadata_geoLocations_geoLocationPlace = TermsFacet(
    field="metadata.geoLocations.geoLocationPlace",
    label=_("metadata/geoLocations/geoLocationPlace.label"),
)

metadata_geoLocations_geoLocationPoint_pointLatitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLatitude",
    label=_("metadata/geoLocations/geoLocationPoint/pointLatitude.label"),
)

metadata_geoLocations_geoLocationPoint_pointLongitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLongitude",
    label=_("metadata/geoLocations/geoLocationPoint/pointLongitude.label"),
)

metadata_languages = VocabularyFacet(
    field="metadata.languages",
    label=_("metadata/languages.label"),
    vocabulary="languages",
)

metadata_methods_cs = TermsFacet(
    field="metadata.methods.cs.keyword", label=_("metadata/methods.label")
)

metadata_methods_en = TermsFacet(
    field="metadata.methods.en.keyword", label=_("metadata/methods.label")
)

metadata_methods_lang = NestedLabeledFacet(
    path="metadata.methods",
    nested_facet=TermsFacet(
        field="metadata.methods.lang", label=_("metadata/methods/lang.label")
    ),
)

metadata_objectIdentifiers_identifier = TermsFacet(
    field="metadata.objectIdentifiers.identifier",
    label=_("metadata/objectIdentifiers/identifier.label"),
)

metadata_objectIdentifiers_scheme = TermsFacet(
    field="metadata.objectIdentifiers.scheme",
    label=_("metadata/objectIdentifiers/scheme.label"),
)

metadata_originalRecord = TermsFacet(
    field="metadata.originalRecord", label=_("metadata/originalRecord.label")
)

metadata_publishers = HierarchyVocabularyFacet(
    field="metadata.publishers",
    label=_("metadata/publishers.label"),
    vocabulary="institutions",
)

metadata_relatedItems_itemContributors_affiliations = HierarchyVocabularyFacet(
    field="metadata.relatedItems.itemContributors.affiliations",
    label=_("metadata/relatedItems/itemContributors/affiliations.label"),
    vocabulary="institutions",
)

metadata_relatedItems_itemContributors_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemContributors.authorityIdentifiers.identifier",
    label=_(
        "metadata/relatedItems/itemContributors/authorityIdentifiers/identifier.label"
    ),
)

metadata_relatedItems_itemContributors_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemContributors.authorityIdentifiers.scheme",
    label=_("metadata/relatedItems/itemContributors/authorityIdentifiers/scheme.label"),
)

metadata_relatedItems_itemContributors_contributorType = VocabularyFacet(
    field="metadata.relatedItems.itemContributors.contributorType",
    label=_("metadata/relatedItems/itemContributors/contributorType.label"),
    vocabulary="contributor-types",
)

metadata_relatedItems_itemContributors_familyName = TermsFacet(
    field="metadata.relatedItems.itemContributors.familyName",
    label=_("metadata/relatedItems/itemContributors/familyName.label"),
)

metadata_relatedItems_itemContributors_fullName = TermsFacet(
    field="metadata.relatedItems.itemContributors.fullName",
    label=_("metadata/relatedItems/itemContributors/fullName.label"),
)

metadata_relatedItems_itemContributors_givenName = TermsFacet(
    field="metadata.relatedItems.itemContributors.givenName",
    label=_("metadata/relatedItems/itemContributors/givenName.label"),
)

metadata_relatedItems_itemContributors_nameType = TermsFacet(
    field="metadata.relatedItems.itemContributors.nameType",
    label=_("metadata/relatedItems/itemContributors/nameType.label"),
)

metadata_relatedItems_itemCreators_affiliations = HierarchyVocabularyFacet(
    field="metadata.relatedItems.itemCreators.affiliations",
    label=_("metadata/relatedItems/itemCreators/affiliations.label"),
    vocabulary="institutions",
)

metadata_relatedItems_itemCreators_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemCreators.authorityIdentifiers.identifier",
    label=_("metadata/relatedItems/itemCreators/authorityIdentifiers/identifier.label"),
)

metadata_relatedItems_itemCreators_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemCreators.authorityIdentifiers.scheme",
    label=_("metadata/relatedItems/itemCreators/authorityIdentifiers/scheme.label"),
)

metadata_relatedItems_itemCreators_familyName = TermsFacet(
    field="metadata.relatedItems.itemCreators.familyName",
    label=_("metadata/relatedItems/itemCreators/familyName.label"),
)

metadata_relatedItems_itemCreators_fullName = TermsFacet(
    field="metadata.relatedItems.itemCreators.fullName",
    label=_("metadata/relatedItems/itemCreators/fullName.label"),
)

metadata_relatedItems_itemCreators_givenName = TermsFacet(
    field="metadata.relatedItems.itemCreators.givenName",
    label=_("metadata/relatedItems/itemCreators/givenName.label"),
)

metadata_relatedItems_itemCreators_nameType = TermsFacet(
    field="metadata.relatedItems.itemCreators.nameType",
    label=_("metadata/relatedItems/itemCreators/nameType.label"),
)

metadata_relatedItems_itemEndPage = TermsFacet(
    field="metadata.relatedItems.itemEndPage",
    label=_("metadata/relatedItems/itemEndPage.label"),
)

metadata_relatedItems_itemIssue = TermsFacet(
    field="metadata.relatedItems.itemIssue",
    label=_("metadata/relatedItems/itemIssue.label"),
)

metadata_relatedItems_itemPIDs_identifier = TermsFacet(
    field="metadata.relatedItems.itemPIDs.identifier",
    label=_("metadata/relatedItems/itemPIDs/identifier.label"),
)

metadata_relatedItems_itemPIDs_scheme = TermsFacet(
    field="metadata.relatedItems.itemPIDs.scheme",
    label=_("metadata/relatedItems/itemPIDs/scheme.label"),
)

metadata_relatedItems_itemPublisher = TermsFacet(
    field="metadata.relatedItems.itemPublisher",
    label=_("metadata/relatedItems/itemPublisher.label"),
)

metadata_relatedItems_itemRelationType = VocabularyFacet(
    field="metadata.relatedItems.itemRelationType",
    label=_("metadata/relatedItems/itemRelationType.label"),
    vocabulary="item-relation-types",
)

metadata_relatedItems_itemResourceType = VocabularyFacet(
    field="metadata.relatedItems.itemResourceType",
    label=_("metadata/relatedItems/itemResourceType.label"),
    vocabulary="resource-types",
)

metadata_relatedItems_itemStartPage = TermsFacet(
    field="metadata.relatedItems.itemStartPage",
    label=_("metadata/relatedItems/itemStartPage.label"),
)

metadata_relatedItems_itemURL = TermsFacet(
    field="metadata.relatedItems.itemURL",
    label=_("metadata/relatedItems/itemURL.label"),
)

metadata_relatedItems_itemVolume = TermsFacet(
    field="metadata.relatedItems.itemVolume",
    label=_("metadata/relatedItems/itemVolume.label"),
)

metadata_relatedItems_itemYear = TermsFacet(
    field="metadata.relatedItems.itemYear",
    label=_("metadata/relatedItems/itemYear.label"),
)

metadata_resourceType = VocabularyFacet(
    field="metadata.resourceType",
    label=_("metadata/resourceType.label"),
    vocabulary="resource-types",
)

metadata_rights = VocabularyFacet(
    field="metadata.rights", label=_("metadata/rights.label"), vocabulary="rights"
)

metadata_series_seriesTitle = TermsFacet(
    field="metadata.series.seriesTitle", label=_("metadata/series/seriesTitle.label")
)

metadata_series_seriesVolume = TermsFacet(
    field="metadata.series.seriesVolume", label=_("metadata/series/seriesVolume.label")
)

metadata_subjectCategories = VocabularyFacet(
    field="metadata.subjectCategories",
    label=_("metadata/subjectCategories.label"),
    vocabulary="subject-categories",
)

metadata_subjects_classificationCode = TermsFacet(
    field="metadata.subjects.classificationCode",
    label=_("metadata/subjects/classificationCode.label"),
)

metadata_subjects_subject_cs = TermsFacet(
    field="metadata.subjects.subject.cs.keyword",
    label=_("metadata/subjects/subject.label"),
)

metadata_subjects_subject_en = TermsFacet(
    field="metadata.subjects.subject.en.keyword",
    label=_("metadata/subjects/subject.label"),
)

metadata_subjects_subject_lang = NestedLabeledFacet(
    path="metadata.subjects.subject",
    nested_facet=TermsFacet(
        field="metadata.subjects.subject.lang",
        label=_("metadata/subjects/subject/lang.label"),
    ),
)

metadata_subjects_subjectScheme = TermsFacet(
    field="metadata.subjects.subjectScheme",
    label=_("metadata/subjects/subjectScheme.label"),
)

metadata_subjects_valueURI = TermsFacet(
    field="metadata.subjects.valueURI", label=_("metadata/subjects/valueURI.label")
)

metadata_systemIdentifiers_identifier = TermsFacet(
    field="metadata.systemIdentifiers.identifier",
    label=_("metadata/systemIdentifiers/identifier.label"),
)

metadata_systemIdentifiers_scheme = TermsFacet(
    field="metadata.systemIdentifiers.scheme",
    label=_("metadata/systemIdentifiers/scheme.label"),
)

metadata_technicalInfo_cs = TermsFacet(
    field="metadata.technicalInfo.cs.keyword", label=_("metadata/technicalInfo.label")
)

metadata_technicalInfo_en = TermsFacet(
    field="metadata.technicalInfo.en.keyword", label=_("metadata/technicalInfo.label")
)

metadata_technicalInfo_lang = NestedLabeledFacet(
    path="metadata.technicalInfo",
    nested_facet=TermsFacet(
        field="metadata.technicalInfo.lang",
        label=_("metadata/technicalInfo/lang.label"),
    ),
)

metadata_version = TermsFacet(
    field="metadata.version", label=_("metadata/version.label")
)



from oarepo_runtime.records.dumpers import SearchDumper

from nr_metadata.data.records.dumpers.edtf import DataEDTFIntervalDumperExt
from nr_metadata.data.records.dumpers.multilingual import MultilingualSearchDumperExt


class DataDumper(SearchDumper):
    """DataRecord opensearch dumper."""

    extensions = [MultilingualSearchDumperExt(), DataEDTFIntervalDumperExt()]



from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2
from invenio_records.systemfields import ConstantField
from invenio_records_resources.records.api import Record as InvenioRecord
from invenio_records_resources.records.systemfields import IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext
from invenio_vocabularies.records.api import Vocabulary
from oarepo_runtime.records.relations import PIDRelation, RelationsField
from oarepo_runtime.records.systemfields import (
    FirstItemSelector,
    PathSelector,
    SyntheticSystemField,
)

from nr_metadata.documents.records.dumpers.dumper import DocumentsDumper
from nr_metadata.documents.records.models import DocumentsMetadata


class DocumentsIdProvider(RecordIdProviderV2):
    pid_type = "dcmnts"


class DocumentsRecord(InvenioRecord):

    model_cls = DocumentsMetadata

    schema = ConstantField("$schema", "local://documents-1.0.0.json")

    index = IndexField("documents-documents-1.0.0")

    pid = PIDField(
        provider=DocumentsIdProvider, context_cls=PIDFieldContext, create=True
    )

    dumper = DocumentsDumper()

    people = SyntheticSystemField(
        PathSelector("metadata.creators", "metadata.contributors"),
        filter=lambda x: x.get("type") == "personal",
        map=lambda x: x.get("fullName"),
        key="syntheticFields.people",
    )

    institutions = SyntheticSystemField(
        PathSelector(
            "metadata.creators.affiliations",
            "metadata.contributors.affiliations",
            "metadata.thesis.degreeGrantors",
        ),
        key="syntheticFields.institutions",
    )

    keywords = SyntheticSystemField(
        PathSelector("metadata.subjects"),
        map=lambda x: x.get("subject", {}).get("value"),
        key="syntheticFields.keywords",
    )

    date = SyntheticSystemField(
        selector=FirstItemSelector("metadata.dateModified", "metadata.dateIssued"),
        key="syntheticFields.date",
    )

    relations = RelationsField(
        accessRights=PIDRelation(
            "metadata.accessRights",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("access-rights"),
        ),
        affiliations=PIDRelation(
            "metadata.contributors.affiliations",
            keys=["id", "title", {"key": "props.ror", "target": "ror"}, "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
        contributorType=PIDRelation(
            "metadata.contributors.contributorType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        Organizational_contributorType=PIDRelation(
            "metadata.contributors.contributorType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        Personal_affiliations=PIDRelation(
            "metadata.creators.affiliations",
            keys=["id", "title", {"key": "props.ror", "target": "ror"}, "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
        country=PIDRelation(
            "metadata.events.eventLocation.country",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("countries"),
        ),
        funder=PIDRelation(
            "metadata.fundingReferences.funder",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("funders"),
        ),
        languages=PIDRelation(
            "metadata.languages",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("languages"),
        ),
        itemContributors_Personal_affiliations=PIDRelation(
            "metadata.relatedItems.itemContributors.affiliations",
            keys=["id", "title", {"key": "props.ror", "target": "ror"}, "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
        Personal_contributorType=PIDRelation(
            "metadata.relatedItems.itemContributors.contributorType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemContributors_Organizational_contributorType=PIDRelation(
            "metadata.relatedItems.itemContributors.contributorType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemCreators_Personal_affiliations=PIDRelation(
            "metadata.relatedItems.itemCreators.affiliations",
            keys=["id", "title", {"key": "props.ror", "target": "ror"}, "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
        itemRelationType=PIDRelation(
            "metadata.relatedItems.itemRelationType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("item-relation-types"),
        ),
        itemResourceType=PIDRelation(
            "metadata.relatedItems.itemResourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        resourceType=PIDRelation(
            "metadata.resourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        rights=PIDRelation(
            "metadata.rights",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("rights"),
        ),
        subjectCategories=PIDRelation(
            "metadata.subjectCategories",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("subject-categories"),
        ),
        degreeGrantors=PIDRelation(
            "metadata.thesis.degreeGrantors",
            keys=["id", "title", "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
        institutions=PIDRelation(
            "syntheticFields.institutions",
            keys=["id", "title", "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("institutions"),
        ),
    )



from invenio_db import db
from invenio_records.models import RecordMetadataBase


class DataMetadata(db.Model, RecordMetadataBase):
    """Model for DataRecord metadata."""

    __tablename__ = "data_metadata"

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}



from invenio_records_resources.services import SearchOptions as InvenioSearchOptions

from . import facets


class DocumentsSearchOptions(InvenioSearchOptions):
    """DocumentsRecord search options."""

    facet_groups = {}

    facets = {
        "metadata_abstract_cs": facets.metadata_abstract_cs,
        "metadata_abstract_en": facets.metadata_abstract_en,
        "metadata_abstract_lang": facets.metadata_abstract_lang,
        "metadata_accessRights": facets.metadata_accessRights,
        "metadata_accessibility_cs": facets.metadata_accessibility_cs,
        "metadata_accessibility_en": facets.metadata_accessibility_en,
        "metadata_accessibility_lang": facets.metadata_accessibility_lang,
        "metadata_additionalTitles_title_cs": facets.metadata_additionalTitles_title_cs,
        "metadata_additionalTitles_title_en": facets.metadata_additionalTitles_title_en,
        "metadata_additionalTitles_title_lang": facets.metadata_additionalTitles_title_lang,
        "metadata_additionalTitles_titleType": facets.metadata_additionalTitles_titleType,
        "metadata_contributors_affiliations": facets.metadata_contributors_affiliations,
        "metadata_contributors_authorityIdentifiers_identifier": facets.metadata_contributors_authorityIdentifiers_identifier,
        "metadata_contributors_authorityIdentifiers_scheme": facets.metadata_contributors_authorityIdentifiers_scheme,
        "metadata_contributors_contributorType": facets.metadata_contributors_contributorType,
        "metadata_contributors_familyName": facets.metadata_contributors_familyName,
        "metadata_contributors_fullName": facets.metadata_contributors_fullName,
        "metadata_contributors_givenName": facets.metadata_contributors_givenName,
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_creators_affiliations": facets.metadata_creators_affiliations,
        "metadata_creators_authorityIdentifiers_identifier": facets.metadata_creators_authorityIdentifiers_identifier,
        "metadata_creators_authorityIdentifiers_scheme": facets.metadata_creators_authorityIdentifiers_scheme,
        "metadata_creators_familyName": facets.metadata_creators_familyName,
        "metadata_creators_fullName": facets.metadata_creators_fullName,
        "metadata_creators_givenName": facets.metadata_creators_givenName,
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_dateAvailable": facets.metadata_dateAvailable,
        "metadata_dateIssued": facets.metadata_dateIssued,
        "metadata_dateModified": facets.metadata_dateModified,
        "metadata_events_eventLocation_country": facets.metadata_events_eventLocation_country,
        "metadata_events_eventLocation_place": facets.metadata_events_eventLocation_place,
        "metadata_externalLocation_externalLocationURL": facets.metadata_externalLocation_externalLocationURL,
        "metadata_fundingReferences_funder": facets.metadata_fundingReferences_funder,
        "metadata_fundingReferences_projectID": facets.metadata_fundingReferences_projectID,
        "metadata_geoLocations_geoLocationPlace": facets.metadata_geoLocations_geoLocationPlace,
        "metadata_geoLocations_geoLocationPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPoint_pointLatitude,
        "metadata_geoLocations_geoLocationPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPoint_pointLongitude,
        "metadata_languages": facets.metadata_languages,
        "metadata_methods_cs": facets.metadata_methods_cs,
        "metadata_methods_en": facets.metadata_methods_en,
        "metadata_methods_lang": facets.metadata_methods_lang,
        "metadata_objectIdentifiers_identifier": facets.metadata_objectIdentifiers_identifier,
        "metadata_objectIdentifiers_scheme": facets.metadata_objectIdentifiers_scheme,
        "metadata_originalRecord": facets.metadata_originalRecord,
        "metadata_relatedItems_itemContributors_affiliations": facets.metadata_relatedItems_itemContributors_affiliations,
        "metadata_relatedItems_itemContributors_authorityIdentifiers_identifier": facets.metadata_relatedItems_itemContributors_authorityIdentifiers_identifier,
        "metadata_relatedItems_itemContributors_authorityIdentifiers_scheme": facets.metadata_relatedItems_itemContributors_authorityIdentifiers_scheme,
        "metadata_relatedItems_itemContributors_contributorType": facets.metadata_relatedItems_itemContributors_contributorType,
        "metadata_relatedItems_itemContributors_familyName": facets.metadata_relatedItems_itemContributors_familyName,
        "metadata_relatedItems_itemContributors_fullName": facets.metadata_relatedItems_itemContributors_fullName,
        "metadata_relatedItems_itemContributors_givenName": facets.metadata_relatedItems_itemContributors_givenName,
        "metadata_relatedItems_itemContributors_nameType": facets.metadata_relatedItems_itemContributors_nameType,
        "metadata_relatedItems_itemCreators_affiliations": facets.metadata_relatedItems_itemCreators_affiliations,
        "metadata_relatedItems_itemCreators_authorityIdentifiers_identifier": facets.metadata_relatedItems_itemCreators_authorityIdentifiers_identifier,
        "metadata_relatedItems_itemCreators_authorityIdentifiers_scheme": facets.metadata_relatedItems_itemCreators_authorityIdentifiers_scheme,
        "metadata_relatedItems_itemCreators_familyName": facets.metadata_relatedItems_itemCreators_familyName,
        "metadata_relatedItems_itemCreators_fullName": facets.metadata_relatedItems_itemCreators_fullName,
        "metadata_relatedItems_itemCreators_givenName": facets.metadata_relatedItems_itemCreators_givenName,
        "metadata_relatedItems_itemCreators_nameType": facets.metadata_relatedItems_itemCreators_nameType,
        "metadata_relatedItems_itemEndPage": facets.metadata_relatedItems_itemEndPage,
        "metadata_relatedItems_itemIssue": facets.metadata_relatedItems_itemIssue,
        "metadata_relatedItems_itemPIDs_identifier": facets.metadata_relatedItems_itemPIDs_identifier,
        "metadata_relatedItems_itemPIDs_scheme": facets.metadata_relatedItems_itemPIDs_scheme,
        "metadata_relatedItems_itemPublisher": facets.metadata_relatedItems_itemPublisher,
        "metadata_relatedItems_itemRelationType": facets.metadata_relatedItems_itemRelationType,
        "metadata_relatedItems_itemResourceType": facets.metadata_relatedItems_itemResourceType,
        "metadata_relatedItems_itemStartPage": facets.metadata_relatedItems_itemStartPage,
        "metadata_relatedItems_itemURL": facets.metadata_relatedItems_itemURL,
        "metadata_relatedItems_itemVolume": facets.metadata_relatedItems_itemVolume,
        "metadata_relatedItems_itemYear": facets.metadata_relatedItems_itemYear,
        "metadata_resourceType": facets.metadata_resourceType,
        "metadata_rights": facets.metadata_rights,
        "metadata_series_seriesTitle": facets.metadata_series_seriesTitle,
        "metadata_series_seriesVolume": facets.metadata_series_seriesVolume,
        "metadata_subjectCategories": facets.metadata_subjectCategories,
        "metadata_subjects_classificationCode": facets.metadata_subjects_classificationCode,
        "metadata_subjects_subject_cs": facets.metadata_subjects_subject_cs,
        "metadata_subjects_subject_en": facets.metadata_subjects_subject_en,
        "metadata_subjects_subject_lang": facets.metadata_subjects_subject_lang,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_valueURI": facets.metadata_subjects_valueURI,
        "metadata_systemIdentifiers_identifier": facets.metadata_systemIdentifiers_identifier,
        "metadata_systemIdentifiers_scheme": facets.metadata_systemIdentifiers_scheme,
        "metadata_technicalInfo_cs": facets.metadata_technicalInfo_cs,
        "metadata_technicalInfo_en": facets.metadata_technicalInfo_en,
        "metadata_technicalInfo_lang": facets.metadata_technicalInfo_lang,
        "metadata_thesis_dateDefended": facets.metadata_thesis_dateDefended,
        "metadata_thesis_defended": facets.metadata_thesis_defended,
        "metadata_thesis_degreeGrantors": facets.metadata_thesis_degreeGrantors,
        "metadata_thesis_studyFields": facets.metadata_thesis_studyFields,
        "metadata_version": facets.metadata_version,
        "syntheticFields_date": facets.syntheticFields_date,
        "syntheticFields_institutions": facets.syntheticFields_institutions,
        "syntheticFields_keywords": facets.syntheticFields_keywords,
        "syntheticFields_people": facets.syntheticFields_people,
        **getattr(InvenioSearchOptions, "facets", {}),
    }



from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links
from invenio_records_resources.services.records.components import DataComponent
from oarepo_runtime.services.config.service import PermissionsPresetsConfigMixin

from nr_metadata.data.records.api import DataRecord
from nr_metadata.data.services.records.permissions import DataPermissionPolicy
from nr_metadata.data.services.records.results import DataRecordItem, DataRecordList
from nr_metadata.data.services.records.schema import NRDataRecordSchema
from nr_metadata.data.services.records.search import DataSearchOptions


class DataServiceConfig(PermissionsPresetsConfigMixin, InvenioRecordServiceConfig):
    """DataRecord service config."""

    result_item_cls = DataRecordItem

    result_list_cls = DataRecordList

    PERMISSIONS_PRESETS = ["everyone"]

    url_prefix = "/nr-metadata-data/"

    base_permission_policy_cls = DataPermissionPolicy

    schema = NRDataRecordSchema

    search = DataSearchOptions

    record_cls = DataRecord

    service_id = "data"

    components = [
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        DataComponent,
    ]

    model = "nr_metadata.data"

    @property
    def links_item(self):
        return {
            "self": RecordLink("{+api}/nr-metadata-data/{id}"),
            "self_html": RecordLink("{+ui}/nr-metadata-data/{id}"),
        }

    @property
    def links_search(self):
        return {
            **pagination_links("{+api}/nr-metadata-data/{?args*}"),
        }



from nr_metadata.common.resources.records.config import CommonResourceConfig
from nr_metadata.common.resources.records.resource import CommonResource
from nr_metadata.common.services.records.config import CommonServiceConfig
from nr_metadata.common.services.records.service import CommonService

COMMON_RECORD_RESOURCE_CONFIG = CommonResourceConfig


COMMON_RECORD_RESOURCE_CLASS = CommonResource


COMMON_RECORD_SERVICE_CONFIG = CommonServiceConfig


COMMON_RECORD_SERVICE_CLASS = CommonService



from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2
from invenio_records.systemfields import ConstantField
from invenio_records_resources.records.api import Record as InvenioRecord
from invenio_records_resources.records.systemfields import IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext

from nr_metadata.datacite.records.dumpers.dumper import DataciteDumper
from nr_metadata.datacite.records.models import DataciteMetadata


class DataciteIdProvider(RecordIdProviderV2):
    pid_type = "dtct"


class DataciteRecord(InvenioRecord):

    model_cls = DataciteMetadata

    schema = ConstantField("$schema", "local://datacite-1.0.0.json")

    index = IndexField("datacite-datacite-1.0.0")

    pid = PIDField(
        provider=DataciteIdProvider, context_cls=PIDFieldContext, create=True
    )

    dumper = DataciteDumper()



from invenio_records_resources.resources import RecordResource


class DataResource(RecordResource):
    """DataRecord resource."""

    # here you can for example redefine
    # create_url_rules function to add your own rules



from invenio_records_resources.services import SearchOptions as InvenioSearchOptions

from . import facets


class DataciteSearchOptions(InvenioSearchOptions):
    """DataciteRecord search options."""

    facet_groups = {}

    facets = {
        "metadata_alternateIdentifiers_alternateIdentifier": facets.metadata_alternateIdentifiers_alternateIdentifier,
        "metadata_alternateIdentifiers_alternateIdentifierType": facets.metadata_alternateIdentifiers_alternateIdentifierType,
        "metadata_container_firstPage": facets.metadata_container_firstPage,
        "metadata_container_title": facets.metadata_container_title,
        "metadata_container_type": facets.metadata_container_type,
        "metadata_contributors_affiliation_affiliationIdentifier": facets.metadata_contributors_affiliation_affiliationIdentifier,
        "metadata_contributors_affiliation_affiliationIdentifierScheme": facets.metadata_contributors_affiliation_affiliationIdentifierScheme,
        "metadata_contributors_affiliation_name": facets.metadata_contributors_affiliation_name,
        "metadata_contributors_affiliation_schemeUri": facets.metadata_contributors_affiliation_schemeUri,
        "metadata_contributors_contributorType": facets.metadata_contributors_contributorType,
        "metadata_contributors_familyName": facets.metadata_contributors_familyName,
        "metadata_contributors_givenName": facets.metadata_contributors_givenName,
        "metadata_contributors_lang": facets.metadata_contributors_lang,
        "metadata_contributors_name": facets.metadata_contributors_name,
        "metadata_contributors_nameIdentifiers_nameIdentifier": facets.metadata_contributors_nameIdentifiers_nameIdentifier,
        "metadata_contributors_nameIdentifiers_nameIdentifierScheme": facets.metadata_contributors_nameIdentifiers_nameIdentifierScheme,
        "metadata_contributors_nameIdentifiers_schemeUri": facets.metadata_contributors_nameIdentifiers_schemeUri,
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_creators_affiliation_affiliationIdentifier": facets.metadata_creators_affiliation_affiliationIdentifier,
        "metadata_creators_affiliation_affiliationIdentifierScheme": facets.metadata_creators_affiliation_affiliationIdentifierScheme,
        "metadata_creators_affiliation_name": facets.metadata_creators_affiliation_name,
        "metadata_creators_affiliation_schemeUri": facets.metadata_creators_affiliation_schemeUri,
        "metadata_creators_familyName": facets.metadata_creators_familyName,
        "metadata_creators_givenName": facets.metadata_creators_givenName,
        "metadata_creators_lang": facets.metadata_creators_lang,
        "metadata_creators_name": facets.metadata_creators_name,
        "metadata_creators_nameIdentifiers_nameIdentifier": facets.metadata_creators_nameIdentifiers_nameIdentifier,
        "metadata_creators_nameIdentifiers_nameIdentifierScheme": facets.metadata_creators_nameIdentifiers_nameIdentifierScheme,
        "metadata_creators_nameIdentifiers_schemeUri": facets.metadata_creators_nameIdentifiers_schemeUri,
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_dates_date": facets.metadata_dates_date,
        "metadata_dates_dateInformation": facets.metadata_dates_dateInformation,
        "metadata_dates_dateType": facets.metadata_dates_dateType,
        "metadata_descriptions_descriptionType": facets.metadata_descriptions_descriptionType,
        "metadata_descriptions_lang": facets.metadata_descriptions_lang,
        "metadata_doi": facets.metadata_doi,
        "metadata_formats": facets.metadata_formats,
        "metadata_fundingReferences_awardNumber": facets.metadata_fundingReferences_awardNumber,
        "metadata_fundingReferences_awardTitle": facets.metadata_fundingReferences_awardTitle,
        "metadata_fundingReferences_awardUri": facets.metadata_fundingReferences_awardUri,
        "metadata_fundingReferences_funderIdentifier": facets.metadata_fundingReferences_funderIdentifier,
        "metadata_fundingReferences_funderIdentifierType": facets.metadata_fundingReferences_funderIdentifierType,
        "metadata_fundingReferences_funderName": facets.metadata_fundingReferences_funderName,
        "metadata_geoLocations_geoLocationBox_eastBoundLongitude": facets.metadata_geoLocations_geoLocationBox_eastBoundLongitude,
        "metadata_geoLocations_geoLocationBox_northBoundLatitude": facets.metadata_geoLocations_geoLocationBox_northBoundLatitude,
        "metadata_geoLocations_geoLocationBox_southBoundLatitude": facets.metadata_geoLocations_geoLocationBox_southBoundLatitude,
        "metadata_geoLocations_geoLocationBox_westBoundLongitude": facets.metadata_geoLocations_geoLocationBox_westBoundLongitude,
        "metadata_geoLocations_geoLocationPlace": facets.metadata_geoLocations_geoLocationPlace,
        "metadata_geoLocations_geoLocationPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPoint_pointLatitude,
        "metadata_geoLocations_geoLocationPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPoint_pointLongitude,
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude,
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude,
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude": facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude,
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude": facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude,
        "metadata_language": facets.metadata_language,
        "metadata_publicationYear": facets.metadata_publicationYear,
        "metadata_publisher_lang": facets.metadata_publisher_lang,
        "metadata_publisher_name": facets.metadata_publisher_name,
        "metadata_publisher_publisherIdentifier": facets.metadata_publisher_publisherIdentifier,
        "metadata_publisher_publisherIdentifierScheme": facets.metadata_publisher_publisherIdentifierScheme,
        "metadata_publisher_schemeURI": facets.metadata_publisher_schemeURI,
        "metadata_relatedIdentifiers_relatedIdentifier": facets.metadata_relatedIdentifiers_relatedIdentifier,
        "metadata_relatedIdentifiers_relatedIdentifierType": facets.metadata_relatedIdentifiers_relatedIdentifierType,
        "metadata_relatedIdentifiers_relatedMetadataScheme": facets.metadata_relatedIdentifiers_relatedMetadataScheme,
        "metadata_relatedIdentifiers_relationType": facets.metadata_relatedIdentifiers_relationType,
        "metadata_relatedIdentifiers_resourceTypeGeneral": facets.metadata_relatedIdentifiers_resourceTypeGeneral,
        "metadata_relatedIdentifiers_schemeType": facets.metadata_relatedIdentifiers_schemeType,
        "metadata_relatedIdentifiers_schemeUri": facets.metadata_relatedIdentifiers_schemeUri,
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifier": facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifier,
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme": facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme,
        "metadata_relatedItems_contributors_affiliation_name": facets.metadata_relatedItems_contributors_affiliation_name,
        "metadata_relatedItems_contributors_affiliation_schemeUri": facets.metadata_relatedItems_contributors_affiliation_schemeUri,
        "metadata_relatedItems_contributors_contributorType": facets.metadata_relatedItems_contributors_contributorType,
        "metadata_relatedItems_contributors_familyName": facets.metadata_relatedItems_contributors_familyName,
        "metadata_relatedItems_contributors_givenName": facets.metadata_relatedItems_contributors_givenName,
        "metadata_relatedItems_contributors_lang": facets.metadata_relatedItems_contributors_lang,
        "metadata_relatedItems_contributors_name": facets.metadata_relatedItems_contributors_name,
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier": facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier,
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme": facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme,
        "metadata_relatedItems_contributors_nameIdentifiers_schemeUri": facets.metadata_relatedItems_contributors_nameIdentifiers_schemeUri,
        "metadata_relatedItems_contributors_nameType": facets.metadata_relatedItems_contributors_nameType,
        "metadata_relatedItems_creators_affiliation_affiliationIdentifier": facets.metadata_relatedItems_creators_affiliation_affiliationIdentifier,
        "metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme": facets.metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme,
        "metadata_relatedItems_creators_affiliation_name": facets.metadata_relatedItems_creators_affiliation_name,
        "metadata_relatedItems_creators_affiliation_schemeUri": facets.metadata_relatedItems_creators_affiliation_schemeUri,
        "metadata_relatedItems_creators_familyName": facets.metadata_relatedItems_creators_familyName,
        "metadata_relatedItems_creators_givenName": facets.metadata_relatedItems_creators_givenName,
        "metadata_relatedItems_creators_lang": facets.metadata_relatedItems_creators_lang,
        "metadata_relatedItems_creators_name": facets.metadata_relatedItems_creators_name,
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifier": facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifier,
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme": facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme,
        "metadata_relatedItems_creators_nameIdentifiers_schemeUri": facets.metadata_relatedItems_creators_nameIdentifiers_schemeUri,
        "metadata_relatedItems_creators_nameType": facets.metadata_relatedItems_creators_nameType,
        "metadata_relatedItems_edition": facets.metadata_relatedItems_edition,
        "metadata_relatedItems_firstPage": facets.metadata_relatedItems_firstPage,
        "metadata_relatedItems_issue": facets.metadata_relatedItems_issue,
        "metadata_relatedItems_lastPage": facets.metadata_relatedItems_lastPage,
        "metadata_relatedItems_number": facets.metadata_relatedItems_number,
        "metadata_relatedItems_numberType": facets.metadata_relatedItems_numberType,
        "metadata_relatedItems_publicationYear": facets.metadata_relatedItems_publicationYear,
        "metadata_relatedItems_publisher": facets.metadata_relatedItems_publisher,
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier": facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier,
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType": facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType,
        "metadata_relatedItems_relatedItemType": facets.metadata_relatedItems_relatedItemType,
        "metadata_relatedItems_titles_lang": facets.metadata_relatedItems_titles_lang,
        "metadata_relatedItems_titles_titleType": facets.metadata_relatedItems_titles_titleType,
        "metadata_relatedItems_volume": facets.metadata_relatedItems_volume,
        "metadata_rightsList_lang": facets.metadata_rightsList_lang,
        "metadata_rightsList_rights": facets.metadata_rightsList_rights,
        "metadata_rightsList_rightsIdentifier": facets.metadata_rightsList_rightsIdentifier,
        "metadata_rightsList_rightsIdentifierScheme": facets.metadata_rightsList_rightsIdentifierScheme,
        "metadata_rightsList_rightsUri": facets.metadata_rightsList_rightsUri,
        "metadata_rightsList_schemeUri": facets.metadata_rightsList_schemeUri,
        "metadata_schemaVersion": facets.metadata_schemaVersion,
        "metadata_sizes": facets.metadata_sizes,
        "metadata_subjects_classificationCode": facets.metadata_subjects_classificationCode,
        "metadata_subjects_lang": facets.metadata_subjects_lang,
        "metadata_subjects_schemeUri": facets.metadata_subjects_schemeUri,
        "metadata_subjects_subject": facets.metadata_subjects_subject,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_valueUri": facets.metadata_subjects_valueUri,
        "metadata_titles_lang": facets.metadata_titles_lang,
        "metadata_titles_titleType": facets.metadata_titles_titleType,
        "metadata_types_resourceType": facets.metadata_types_resourceType,
        "metadata_types_resourceTypeGeneral": facets.metadata_types_resourceTypeGeneral,
        "metadata_url": facets.metadata_url,
        "metadata_version": facets.metadata_version,
        **getattr(InvenioSearchOptions, "facets", {}),
        **getattr(InvenioSearchOptions, "facets", {}),
        "metadata_alternateIdentifiers_alternateIdentifier": facets.metadata_alternateIdentifiers_alternateIdentifier,
        "metadata_alternateIdentifiers_alternateIdentifierType": facets.metadata_alternateIdentifiers_alternateIdentifierType,
        "metadata_container_firstPage": facets.metadata_container_firstPage,
        "metadata_container_title": facets.metadata_container_title,
        "metadata_container_type": facets.metadata_container_type,
        "metadata_contributors_affiliation_affiliationIdentifier": facets.metadata_contributors_affiliation_affiliationIdentifier,
        "metadata_contributors_affiliation_affiliationIdentifierScheme": facets.metadata_contributors_affiliation_affiliationIdentifierScheme,
        "metadata_contributors_affiliation_name": facets.metadata_contributors_affiliation_name,
        "metadata_contributors_affiliation_schemeUri": facets.metadata_contributors_affiliation_schemeUri,
        "metadata_contributors_contributorType": facets.metadata_contributors_contributorType,
        "metadata_contributors_familyName": facets.metadata_contributors_familyName,
        "metadata_contributors_givenName": facets.metadata_contributors_givenName,
        "metadata_contributors_lang": facets.metadata_contributors_lang,
        "metadata_contributors_name": facets.metadata_contributors_name,
        "metadata_contributors_nameIdentifiers_nameIdentifier": facets.metadata_contributors_nameIdentifiers_nameIdentifier,
        "metadata_contributors_nameIdentifiers_nameIdentifierScheme": facets.metadata_contributors_nameIdentifiers_nameIdentifierScheme,
        "metadata_contributors_nameIdentifiers_schemeUri": facets.metadata_contributors_nameIdentifiers_schemeUri,
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_creators_affiliation_affiliationIdentifier": facets.metadata_creators_affiliation_affiliationIdentifier,
        "metadata_creators_affiliation_affiliationIdentifierScheme": facets.metadata_creators_affiliation_affiliationIdentifierScheme,
        "metadata_creators_affiliation_name": facets.metadata_creators_affiliation_name,
        "metadata_creators_affiliation_schemeUri": facets.metadata_creators_affiliation_schemeUri,
        "metadata_creators_familyName": facets.metadata_creators_familyName,
        "metadata_creators_givenName": facets.metadata_creators_givenName,
        "metadata_creators_lang": facets.metadata_creators_lang,
        "metadata_creators_name": facets.metadata_creators_name,
        "metadata_creators_nameIdentifiers_nameIdentifier": facets.metadata_creators_nameIdentifiers_nameIdentifier,
        "metadata_creators_nameIdentifiers_nameIdentifierScheme": facets.metadata_creators_nameIdentifiers_nameIdentifierScheme,
        "metadata_creators_nameIdentifiers_schemeUri": facets.metadata_creators_nameIdentifiers_schemeUri,
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_dates_date": facets.metadata_dates_date,
        "metadata_dates_dateInformation": facets.metadata_dates_dateInformation,
        "metadata_dates_dateType": facets.metadata_dates_dateType,
        "metadata_descriptions_descriptionType": facets.metadata_descriptions_descriptionType,
        "metadata_descriptions_lang": facets.metadata_descriptions_lang,
        "metadata_doi": facets.metadata_doi,
        "metadata_formats": facets.metadata_formats,
        "metadata_fundingReferences_awardNumber": facets.metadata_fundingReferences_awardNumber,
        "metadata_fundingReferences_awardTitle": facets.metadata_fundingReferences_awardTitle,
        "metadata_fundingReferences_awardUri": facets.metadata_fundingReferences_awardUri,
        "metadata_fundingReferences_funderIdentifier": facets.metadata_fundingReferences_funderIdentifier,
        "metadata_fundingReferences_funderIdentifierType": facets.metadata_fundingReferences_funderIdentifierType,
        "metadata_fundingReferences_funderName": facets.metadata_fundingReferences_funderName,
        "metadata_geoLocations_geoLocationBox_eastBoundLongitude": facets.metadata_geoLocations_geoLocationBox_eastBoundLongitude,
        "metadata_geoLocations_geoLocationBox_northBoundLatitude": facets.metadata_geoLocations_geoLocationBox_northBoundLatitude,
        "metadata_geoLocations_geoLocationBox_southBoundLatitude": facets.metadata_geoLocations_geoLocationBox_southBoundLatitude,
        "metadata_geoLocations_geoLocationBox_westBoundLongitude": facets.metadata_geoLocations_geoLocationBox_westBoundLongitude,
        "metadata_geoLocations_geoLocationPlace": facets.metadata_geoLocations_geoLocationPlace,
        "metadata_geoLocations_geoLocationPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPoint_pointLatitude,
        "metadata_geoLocations_geoLocationPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPoint_pointLongitude,
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude,
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude,
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude": facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude,
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude": facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude,
        "metadata_language": facets.metadata_language,
        "metadata_publicationYear": facets.metadata_publicationYear,
        "metadata_publisher_lang": facets.metadata_publisher_lang,
        "metadata_publisher_name": facets.metadata_publisher_name,
        "metadata_publisher_publisherIdentifier": facets.metadata_publisher_publisherIdentifier,
        "metadata_publisher_publisherIdentifierScheme": facets.metadata_publisher_publisherIdentifierScheme,
        "metadata_publisher_schemeURI": facets.metadata_publisher_schemeURI,
        "metadata_relatedIdentifiers_relatedIdentifier": facets.metadata_relatedIdentifiers_relatedIdentifier,
        "metadata_relatedIdentifiers_relatedIdentifierType": facets.metadata_relatedIdentifiers_relatedIdentifierType,
        "metadata_relatedIdentifiers_relatedMetadataScheme": facets.metadata_relatedIdentifiers_relatedMetadataScheme,
        "metadata_relatedIdentifiers_relationType": facets.metadata_relatedIdentifiers_relationType,
        "metadata_relatedIdentifiers_resourceTypeGeneral": facets.metadata_relatedIdentifiers_resourceTypeGeneral,
        "metadata_relatedIdentifiers_schemeType": facets.metadata_relatedIdentifiers_schemeType,
        "metadata_relatedIdentifiers_schemeUri": facets.metadata_relatedIdentifiers_schemeUri,
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifier": facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifier,
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme": facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme,
        "metadata_relatedItems_contributors_affiliation_name": facets.metadata_relatedItems_contributors_affiliation_name,
        "metadata_relatedItems_contributors_affiliation_schemeUri": facets.metadata_relatedItems_contributors_affiliation_schemeUri,
        "metadata_relatedItems_contributors_contributorType": facets.metadata_relatedItems_contributors_contributorType,
        "metadata_relatedItems_contributors_familyName": facets.metadata_relatedItems_contributors_familyName,
        "metadata_relatedItems_contributors_givenName": facets.metadata_relatedItems_contributors_givenName,
        "metadata_relatedItems_contributors_lang": facets.metadata_relatedItems_contributors_lang,
        "metadata_relatedItems_contributors_name": facets.metadata_relatedItems_contributors_name,
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier": facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier,
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme": facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme,
        "metadata_relatedItems_contributors_nameIdentifiers_schemeUri": facets.metadata_relatedItems_contributors_nameIdentifiers_schemeUri,
        "metadata_relatedItems_contributors_nameType": facets.metadata_relatedItems_contributors_nameType,
        "metadata_relatedItems_creators_affiliation_affiliationIdentifier": facets.metadata_relatedItems_creators_affiliation_affiliationIdentifier,
        "metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme": facets.metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme,
        "metadata_relatedItems_creators_affiliation_name": facets.metadata_relatedItems_creators_affiliation_name,
        "metadata_relatedItems_creators_affiliation_schemeUri": facets.metadata_relatedItems_creators_affiliation_schemeUri,
        "metadata_relatedItems_creators_familyName": facets.metadata_relatedItems_creators_familyName,
        "metadata_relatedItems_creators_givenName": facets.metadata_relatedItems_creators_givenName,
        "metadata_relatedItems_creators_lang": facets.metadata_relatedItems_creators_lang,
        "metadata_relatedItems_creators_name": facets.metadata_relatedItems_creators_name,
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifier": facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifier,
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme": facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme,
        "metadata_relatedItems_creators_nameIdentifiers_schemeUri": facets.metadata_relatedItems_creators_nameIdentifiers_schemeUri,
        "metadata_relatedItems_creators_nameType": facets.metadata_relatedItems_creators_nameType,
        "metadata_relatedItems_edition": facets.metadata_relatedItems_edition,
        "metadata_relatedItems_firstPage": facets.metadata_relatedItems_firstPage,
        "metadata_relatedItems_issue": facets.metadata_relatedItems_issue,
        "metadata_relatedItems_lastPage": facets.metadata_relatedItems_lastPage,
        "metadata_relatedItems_number": facets.metadata_relatedItems_number,
        "metadata_relatedItems_numberType": facets.metadata_relatedItems_numberType,
        "metadata_relatedItems_publicationYear": facets.metadata_relatedItems_publicationYear,
        "metadata_relatedItems_publisher": facets.metadata_relatedItems_publisher,
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier": facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier,
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType": facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType,
        "metadata_relatedItems_relatedItemType": facets.metadata_relatedItems_relatedItemType,
        "metadata_relatedItems_titles_lang": facets.metadata_relatedItems_titles_lang,
        "metadata_relatedItems_titles_titleType": facets.metadata_relatedItems_titles_titleType,
        "metadata_relatedItems_volume": facets.metadata_relatedItems_volume,
        "metadata_rightsList_lang": facets.metadata_rightsList_lang,
        "metadata_rightsList_rights": facets.metadata_rightsList_rights,
        "metadata_rightsList_rightsIdentifier": facets.metadata_rightsList_rightsIdentifier,
        "metadata_rightsList_rightsIdentifierScheme": facets.metadata_rightsList_rightsIdentifierScheme,
        "metadata_rightsList_rightsUri": facets.metadata_rightsList_rightsUri,
        "metadata_rightsList_schemeUri": facets.metadata_rightsList_schemeUri,
        "metadata_schemaVersion": facets.metadata_schemaVersion,
        "metadata_sizes": facets.metadata_sizes,
        "metadata_subjects_classificationCode": facets.metadata_subjects_classificationCode,
        "metadata_subjects_lang": facets.metadata_subjects_lang,
        "metadata_subjects_schemeUri": facets.metadata_subjects_schemeUri,
        "metadata_subjects_subject": facets.metadata_subjects_subject,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_valueUri": facets.metadata_subjects_valueUri,
        "metadata_titles_lang": facets.metadata_titles_lang,
        "metadata_titles_titleType": facets.metadata_titles_titleType,
        "metadata_types_resourceType": facets.metadata_types_resourceType,
        "metadata_types_resourceTypeGeneral": facets.metadata_types_resourceTypeGeneral,
        "metadata_url": facets.metadata_url,
        "metadata_version": facets.metadata_version,
        **getattr(InvenioSearchOptions, "facets", {}),
    }

    facet_groups = {}

    facet_groups = {}

    facet_groups = {}

    facet_groups = {}

    facet_groups = {}

    facet_groups = {}



from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links
from invenio_records_resources.services.records.components import DataComponent
from oarepo_runtime.services.config.service import PermissionsPresetsConfigMixin

from nr_metadata.datacite.records.api import DataciteRecord
from nr_metadata.datacite.services.records.permissions import DatacitePermissionPolicy
from nr_metadata.datacite.services.records.results import (
    DataciteRecordItem,
    DataciteRecordList,
)
from nr_metadata.datacite.services.records.schema import DataCiteRecordSchema
from nr_metadata.datacite.services.records.search import DataciteSearchOptions


class DataciteServiceConfig(PermissionsPresetsConfigMixin, InvenioRecordServiceConfig):
    """DataciteRecord service config."""

    result_item_cls = DataciteRecordItem

    result_list_cls = DataciteRecordList

    PERMISSIONS_PRESETS = ["everyone"]

    url_prefix = "/nr-metadata-datacite/"

    base_permission_policy_cls = DatacitePermissionPolicy

    schema = DataCiteRecordSchema

    search = DataciteSearchOptions

    record_cls = DataciteRecord

    service_id = "datacite"

    components = [
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        DataComponent,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
        *PermissionsPresetsConfigMixin.components,
        *InvenioRecordServiceConfig.components,
    ]

    model = "nr_metadata.datacite"

    @property
    def links_item(self):
        return {
            "self": RecordLink("{+api}/nr-metadata-datacite/{id}"),
            "self_html": RecordLink("{+ui}/nr-metadata-datacite/{id}"),
        }

    @property
    def links_search(self):
        return {
            **pagination_links("{+api}/nr-metadata-datacite/{?args*}"),
        }



import marshmallow as ma
from edtf import Interval as EDTFInterval
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import fields as ma_fields
from marshmallow.fields import String
from marshmallow_utils.fields import TrimmedString
from oarepo_runtime.services.schema.marshmallow import BaseRecordSchema, DictOnlySchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_date,
)
from oarepo_vocabularies.services.schema import HierarchySchema

from nr_metadata.common.services.records.schema_common import (
    AdditionalTitlesSchema,
    NRCommonMetadataSchema,
)
from nr_metadata.common.services.records.schema_datatypes import (
    NREventSchema,
    NRFundingReferenceSchema,
    NRGeoLocationSchema,
    NRRelatedItemSchema,
    NRSeriesSchema,
    NRSubjectSchema,
)
from nr_metadata.schema.identifiers import (
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class NRDataRecordSchema(BaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma_fields.Nested(lambda: NRDataMetadataSchema())


class NRDataMetadataSchema(NRCommonMetadataSchema):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )

    dateCollected = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    dateCreated = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    dateValidTo = ma_fields.String(validate=[validate_date("%Y-%m-%d")])

    dateWithdrawn = ma_fields.Nested(lambda: DateWithdrawnSchema())

    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: NRFundingReferenceSchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierSchema())
    )

    publishers = ma_fields.List(
        ma_fields.Nested(lambda: PublishersItemSchema()),
        validate=[ma.validate.Length(min=1)],
    )

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))

    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )


class DateWithdrawnSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    dateInformation = ma_fields.String()

    type = ma_fields.String(validate=[validate_date("%Y-%m-%d")])


class PublishersItemSchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchySchema())

    ror = ma_fields.String()

    title = i18n_strings



import marshmallow as ma
from edtf import Date as EDTFDate
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import fields as ma_fields
from marshmallow.fields import String
from marshmallow_utils.fields import TrimmedString
from oarepo_runtime.services.schema.marshmallow import BaseRecordSchema, DictOnlySchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_date,
)
from oarepo_vocabularies.services.schema import HierarchySchema

from nr_metadata.common.services.records.schema_common import (
    AdditionalTitlesSchema,
    NRCommonMetadataSchema,
)
from nr_metadata.common.services.records.schema_datatypes import (
    NREventSchema,
    NRExternalLocationSchema,
    NRFundingReferenceSchema,
    NRGeoLocationSchema,
    NRRelatedItemSchema,
    NRSeriesSchema,
    NRSubjectSchema,
)
from nr_metadata.schema.identifiers import (
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class NRDocumentRecordSchema(BaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma_fields.Nested(lambda: NRDocumentMetadataSchema())

    syntheticFields = ma_fields.Nested(lambda: NRDocumentSyntheticFieldsSchema())


class NRDocumentMetadataSchema(NRCommonMetadataSchema):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )

    dateModified = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFDate,))]
    )

    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))

    externalLocation = ma_fields.Nested(lambda: NRExternalLocationSchema())

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: NRFundingReferenceSchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierSchema())
    )

    publishers = ma_fields.List(ma_fields.String())

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))

    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )

    thesis = ma_fields.Nested(lambda: NRThesisSchema())


class NRThesisSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    dateDefended = ma_fields.String(validate=[validate_date("%Y-%m-%d")])

    defended = ma_fields.Boolean()

    degreeGrantors = ma_fields.List(ma_fields.Nested(lambda: NRDegreeGrantorSchema()))

    studyFields = ma_fields.List(ma_fields.String())


class InstitutionsSchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class NRDegreeGrantorSchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class NRDocumentSyntheticFieldsSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE



import marshmallow as ma
from edtf import Date as EDTFDate
from marshmallow import fields as ma_fields
from marshmallow.validate import OneOf
from marshmallow_utils.fields import TrimmedString
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.validation import CachedMultilayerEDTFValidator


class GeoLocationSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    geoLocationBox = ma_fields.Nested(lambda: GeoLocationBoxSchema())

    geoLocationPlace = ma_fields.String()

    geoLocationPoint = ma_fields.Nested(lambda: GeoLocationPointSchema())

    geoLocationPolygons = ma_fields.List(
        ma_fields.Nested(lambda: GeoLocationPolygonSchema())
    )


class RelatedItemSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    contributors = ma_fields.List(ma_fields.Nested(lambda: ContributorSchema()))

    creators = ma_fields.List(ma_fields.Nested(lambda: CreatorSchema()))

    edition = ma_fields.String()

    firstPage = ma_fields.String()

    issue = ma_fields.String()

    lastPage = ma_fields.String()

    number = ma_fields.String()

    numberType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                    "Article",
                    "Chapter",
                    "Report",
                    "Other",
                ]
            )
        ]
    )

    publicationYear = ma_fields.String()

    publisher = ma_fields.String()

    relatedItemIdentifier = ma_fields.Nested(
        lambda: RelatedItemIdentifierSchema(), required=True
    )

    relatedItemType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                ]
            )
        ],
    )

    titles = ma_fields.List(
        ma_fields.Nested(lambda: RelatedItemTitleSchema()), required=True
    )

    volume = ma_fields.String()


class ContributorSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliation = ma_fields.List(ma_fields.Nested(lambda: AffiliationSchema()))

    contributorType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                    "ContactPerson",
                    "DataCollector",
                    "DataCurator",
                    "DataManager",
                    "Distributor",
                    "Editor",
                    "HostingInstitution",
                    "Producer",
                    "ProjectLeader",
                    "ProjectManager",
                    "ProjectMember",
                    "RegistrationAgency",
                    "RegistrationAuthority",
                    "RelatedPerson",
                    "Researcher",
                    "ResearchGroup",
                    "RightsHolder",
                    "Sponsor",
                    "Supervisor",
                    "WorkPackageLeader",
                    "Other",
                ]
            )
        ],
    )

    familyName = ma_fields.String()

    givenName = ma_fields.String()

    lang = ma_fields.String()

    name = ma_fields.String(required=True)

    nameIdentifiers = ma_fields.List(ma_fields.Nested(lambda: NameIdentifierSchema()))

    nameType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                ]
            )
        ]
    )


class CreatorSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliation = ma_fields.List(ma_fields.Nested(lambda: AffiliationSchema()))

    familyName = ma_fields.String()

    givenName = ma_fields.String()

    lang = ma_fields.String()

    name = ma_fields.String(required=True)

    nameIdentifiers = ma_fields.List(ma_fields.Nested(lambda: NameIdentifierSchema()))

    nameType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                    "Organizational",
                    "Personal",
                ]
            )
        ]
    )


class GeoLocationPolygonSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    inPolygonPoint = ma_fields.Nested(lambda: GeoLocationPointSchema())

    polygonPoints = ma_fields.List(
        ma_fields.Nested(lambda: GeoLocationPointSchema()), required=True
    )


class AffiliationSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    affiliationIdentifier = ma_fields.String()

    affiliationIdentifierScheme = ma_fields.String()

    name = ma_fields.String(required=True)

    schemeUri = ma_fields.String()


class AlternateIdentifierSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    alternateIdentifier = ma_fields.String(required=True)

    alternateIdentifierType = ma_fields.String(required=True)


class ContainerSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    firstPage = ma_fields.String()

    title = ma_fields.String()

    type = ma_fields.String()


class DateSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    date = TrimmedString(
        required=True, validate=[CachedMultilayerEDTFValidator(types=(EDTFDate,))]
    )

    dateInformation = ma_fields.String()

    dateType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                    "Accepted",
                    "Available",
                    "Copyrighted",
                    "Collected",
                    "Created",
                    "Issued",
                    "Submitted",
                    "Updated",
                    "Valid",
                    "Withdrawn",
                    "Other",
                ]
            )
        ],
    )


class DescriptionSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    description = ma_fields.String(required=True)

    descriptionType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                    "Abstract",
                    "Methods",
                    "SeriesInformation",
                    "TableOfContents",
                    "TechnicalInfo",
                    "Other",
                ]
            )
        ],
    )

    lang = ma_fields.String()


class FundingReferenceSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    awardNumber = ma_fields.String()

    awardTitle = ma_fields.String()

    awardUri = ma_fields.String()

    funderIdentifier = ma_fields.String()

    funderIdentifierType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                    "ISNI",
                    "GRID",
                    "Crossref Funder ID",
                    "ROR",
                    "Other",
                ]
            )
        ]
    )

    funderName = ma_fields.String(required=True)


class GeoLocationBoxSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    eastBoundLongitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-180.0, max=180.0)]
    )

    northBoundLatitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-90.0, max=90.0)]
    )

    southBoundLatitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-90.0, max=90.0)]
    )

    westBoundLongitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-180.0, max=180.0)]
    )


class GeoLocationPointSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    pointLatitude = ma_fields.Float(validate=[ma.validate.Range(min=-90.0, max=90.0)])

    pointLongitude = ma_fields.Float(
        validate=[ma.validate.Range(min=-180.0, max=180.0)]
    )


class NameIdentifierSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    nameIdentifier = ma_fields.String(required=True)

    nameIdentifierScheme = ma_fields.String(required=True)

    schemeUri = ma_fields.String()


class PublisherSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    lang = ma_fields.String()

    name = ma_fields.String(required=True)

    publisherIdentifier = ma_fields.String()

    publisherIdentifierScheme = ma_fields.String()

    schemeURI = ma_fields.String()


class RelatedIdentifierSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    relatedIdentifier = ma_fields.String()

    relatedIdentifierType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                ]
            )
        ]
    )

    relatedMetadataScheme = ma_fields.String()

    relationType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                    "IsCitedBy",
                    "Cites",
                    "IsCollectedBy",
                    "Collects",
                    "IsSupplementTo",
                    "IsSupplementedBy",
                    "IsContinuedBy",
                    "Continues",
                    "IsDescribedBy",
                    "Describes",
                    "HasMetadata",
                    "IsMetadataFor",
                    "HasVersion",
                    "IsVersionOf",
                    "IsNewVersionOf",
                    "IsPartOf",
                    "IsPreviousVersionOf",
                    "IsPublishedIn",
                    "HasPart",
                    "IsReferencedBy",
                    "References",
                    "IsDocumentedBy",
                    "Documents",
                    "IsCompiledBy",
                    "Compiles",
                    "IsVariantFormOf",
                    "IsOriginalFormOf",
                    "IsIdenticalTo",
                    "IsReviewedBy",
                    "Reviews",
                    "IsDerivedFrom",
                    "IsSourceOf",
                    "IsRequiredBy",
                    "Requires",
                    "IsObsoletedBy",
                    "Obsoletes",
                ]
            )
        ],
    )

    resourceTypeGeneral = ma_fields.String(
        validate=[
            OneOf(
                [
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                ]
            )
        ]
    )

    schemeType = ma_fields.String()

    schemeUri = ma_fields.String()


class RelatedItemIdentifierSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    relatedItemIdentifier = ma_fields.String(required=True)

    relatedItemIdentifierType = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                    "ARK",
                    "arXiv",
                    "bibcode",
                    "DOI",
                    "EAN13",
                    "EISSN",
                    "Handle",
                    "IGSN",
                    "ISBN",
                    "ISSN",
                    "ISTC",
                    "LISSN",
                    "LSID",
                    "PMID",
                    "PURL",
                    "UPC",
                    "URL",
                    "URN",
                    "w3id",
                ]
            )
        ],
    )


class RelatedItemTitleSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    lang = ma_fields.String()

    title = ma_fields.String(required=True)

    titleType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                ]
            )
        ]
    )


class ResourceTypeSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    resourceType = ma_fields.String()

    resourceTypeGeneral = ma_fields.String(
        required=True,
        validate=[
            OneOf(
                [
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                    "Audiovisual",
                    "Book",
                    "BookChapter",
                    "Collection",
                    "ComputationalNotebook",
                    "ConferencePaper",
                    "ConferenceProceeding",
                    "DataPaper",
                    "Dataset",
                    "Dissertation",
                    "Event",
                    "Image",
                    "Instrument",
                    "InteractiveResource",
                    "Journal",
                    "JournalArticle",
                    "Model",
                    "OutputManagementPlan",
                    "PeerReview",
                    "PhysicalObject",
                    "Preprint",
                    "Report",
                    "Service",
                    "Software",
                    "Sound",
                    "Standard",
                    "StudyRegistration",
                    "Text",
                    "Workflow",
                    "Other",
                ]
            )
        ],
    )


class RightsSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    lang = ma_fields.String()

    rights = ma_fields.String()

    rightsIdentifier = ma_fields.String()

    rightsIdentifierScheme = ma_fields.String()

    rightsUri = ma_fields.String()

    schemeUri = ma_fields.String()


class SubjectSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    classificationCode = ma_fields.String()

    lang = ma_fields.String()

    schemeUri = ma_fields.String()

    subject = ma_fields.String(required=True)

    subjectScheme = ma_fields.String()

    valueUri = ma_fields.String()


class TitleSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    lang = ma_fields.String()

    title = ma_fields.String(required=True)

    titleType = ma_fields.String(
        validate=[
            OneOf(
                [
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                    "AlternativeTitle",
                    "Subtitle",
                    "TranslatedTitle",
                    "Other",
                ]
            )
        ]
    )



from oarepo_runtime.records.dumpers import SearchDumper

from nr_metadata.datacite.records.dumpers.edtf import DataciteEDTFIntervalDumperExt
from nr_metadata.datacite.records.dumpers.multilingual import (
    MultilingualSearchDumperExt,
)


class DataciteDumper(SearchDumper):
    """DataciteRecord opensearch dumper."""

    extensions = [DataciteEDTFIntervalDumperExt(), MultilingualSearchDumperExt()]
    extensions = [DataciteEDTFIntervalDumperExt(), MultilingualSearchDumperExt()]
    extensions = [DataciteEDTFIntervalDumperExt(), MultilingualSearchDumperExt()]
    extensions = [DataciteEDTFIntervalDumperExt(), MultilingualSearchDumperExt()]
    extensions = [MultilingualSearchDumperExt(), DataciteEDTFIntervalDumperExt()]
    extensions = [MultilingualSearchDumperExt(), DataciteEDTFIntervalDumperExt()]
    extensions = [MultilingualSearchDumperExt(), DataciteEDTFIntervalDumperExt()]



import marshmallow as ma


class NRSubjectListField(ma.fields.List):
    def __init__(self, cls_or_instance, **kwargs):
        super().__init__(ma.fields.Raw(), **kwargs)

    def _serialize(self, value, attr=None, obj=None, **kwargs):
        raw = super()._serialize(value, attr, obj, **kwargs)
        ret = []
        for r in raw:
            for rr in r["subject"]:
                ret.append(
                    {
                        **r,
                        "subject": rr,
                    }
                )
        return ret



import marshmallow as ma
from marshmallow import fields as ma_fields
from marshmallow.fields import String
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.ui import (
    InvenioUISchema,
    LocalizedDate,
    LocalizedEDTF,
)
from oarepo_vocabularies.services.ui_schema import (
    HierarchyUISchema,
    VocabularyI18nStrUIField,
)

from nr_metadata.common.services.records.ui_schema_common import (
    AdditionalTitlesUISchema,
    NRCommonMetadataUISchema,
)
from nr_metadata.common.services.records.ui_schema_datatypes import (
    NREventUISchema,
    NRExternalLocationUISchema,
    NRFundingReferenceUISchema,
    NRGeoLocationUISchema,
    NRRelatedItemUISchema,
    NRSeriesUISchema,
    NRSubjectUISchema,
)
from nr_metadata.ui_schema.identifiers import (
    NRObjectIdentifierUISchema,
    NRSystemIdentifierUISchema,
)
from nr_metadata.ui_schema.subjects import NRSubjectListField


class NRDocumentRecordUISchema(InvenioUISchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma_fields.Nested(lambda: NRDocumentMetadataUISchema())

    syntheticFields = ma_fields.Nested(lambda: NRDocumentSyntheticFieldsUISchema())


class NRDocumentMetadataUISchema(NRCommonMetadataUISchema):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesUISchema())
    )

    dateModified = LocalizedEDTF()

    events = ma_fields.List(ma_fields.Nested(lambda: NREventUISchema()))

    externalLocation = ma_fields.Nested(lambda: NRExternalLocationUISchema())

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: NRFundingReferenceUISchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationUISchema()))

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierUISchema())
    )

    publishers = ma_fields.List(ma_fields.String())

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemUISchema()))

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesUISchema()))

    subjects = NRSubjectListField(ma_fields.Nested(lambda: NRSubjectUISchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierUISchema())
    )

    thesis = ma_fields.Nested(lambda: NRThesisUISchema())


class NRDocumentSyntheticFieldsUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    date = LocalizedDate()

    institutions = ma_fields.Nested(lambda: InstitutionsUISchema())

    keywords = ma_fields.String()

    people = ma_fields.String()


class NRThesisUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    dateDefended = LocalizedDate()

    defended = ma_fields.Boolean()

    degreeGrantors = ma_fields.List(ma_fields.Nested(lambda: NRDegreeGrantorUISchema()))

    studyFields = ma_fields.List(ma_fields.String())


class InstitutionsUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchyUISchema())

    title = VocabularyI18nStrUIField()


class NRDegreeGrantorUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchyUISchema())

    title = VocabularyI18nStrUIField()
